/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.taglib;

import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.db.DBTableModel;
import org.opengion.hayabusa.report.DBTableReport;
import org.opengion.fukurou.util.FileUtil;
import org.opengion.fukurou.util.Shell;
import org.opengion.fukurou.util.ToString;						// 6.1.1.0 (2015/01/17)
import org.opengion.fukurou.util.StringUtil ;

import static org.opengion.fukurou.util.StringUtil.nval ;
import static org.opengion.fukurou.util.HybsConst.BR;		// 6.1.0.0 (2014/12/26) refactoring
import static org.opengion.fukurou.util.HybsConst.FS;		// 6.1.0.0 (2014/12/26) refactoring

import java.io.File;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;

/**
 * 検索結果の DBTableModelオブジェクトをレポート形式に変換するタグです。
 *
 * データ(DBTableModel)と、コントローラ(DBTableReport クラス)を与えて、
 * 外部からコントロールすることで、各種形式で データ(DBTableModel)を表示させることが
 * 可能です。
 *
 * @og.formSample
 * ●形式：&lt;og:report fileURL="[･･･]" listId="[･･･]" ･･･ /&gt;
 * ●body：なし
 *
 * ●Tag定義：
 *   &lt;og:report
 *       listId           ○【TAG】帳票IDを指定します(必須)。
 *       fileURL            【TAG】雛型のHTMLファイルの保存してある ディレクトリを指定します
 *       programFile        【TAG】HTMLファイルをEXCEL変換する場合に使用するBATファイルを指定します
 *       outFileURL         【TAG】出力HTMLファイルの保存してあるディレクトリを指定します
 *       outFilename      ○【TAG】ファイルを作成するときの出力ファイル名をセットします(必須)。
 *       headerKeys         【TAG】固定部の{&#064;KEY} の KEY 部分をCSV形式で複数指定します
 *       headerVals         【TAG】固定部のKEY に対応する値をCSV形式で複数指定します
 *       footerKeys         【TAG】繰り返し部の終了後に表示する key 部分をCSV形式で複数指定します
 *       footerVals         【TAG】固定部のKEY に対応する値をCSV形式で複数指定します
 *       pageEndCut         【TAG】ボディー部(繰り返し部)がなくなったときに、それ以降を表示するかどうか[true/false]を指定します(初期値:true)
 *       reportClass        【TAG】実際に書き出すクラス名の略称(DBTableReport_**** の ****)をセットします(初期値:HTML)
 *       language           【TAG】タグ内部で使用する言語コード[ja/en/zh/…]を指定します
 *       scope              【TAG】キャッシュする場合のスコープ[request/page/session/applicaton]を指定します(初期値:session)
 *       tableId            【TAG】(通常使いません)sessionから所得する DBTableModelオブジェクトの ID
 *       debug              【TAG】デバッグ情報を出力するかどうか[true/false]を指定します(初期値:false)
 *   /&gt;
 *
 * ●使用例
 *
 * @og.group その他出力
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class ReportTableTag extends CommonTagSupport {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0.0 (2007/11/28)" ;

	private static final long serialVersionUID = 400020071128L ;

	// 印刷時に使用するテンポラリフォルダ名
	private final String REPORT_URL =
		nval( HybsSystem.sys( "REPORT_FILE_URL" ) ,
				 HybsSystem.sys( "FILE_URL" ) + "REPORT/" ) ;

	// 3.8.0.4 (2005/08/08) 印刷時に使用するシステムID
	private static final String SYSTEM_ID =HybsSystem.sys( "SYSTEM_ID" );

	// 3.8.0.4 (2005/08/08) 帳票出力に準拠した方式に変更
	private final int TIMEOUT = HybsSystem.sysInt( "REPORT_DAEMON_TIMEOUT" );

	private final String  BASE_URL	= HybsSystem.sys( "FILE_URL" );

	private String[]  headerKeys	;				// 固定部の{@KEY} の KEY 部分を指定する。カンマで複数指定できる。
	private String[]  headerVals	;				// 固定部のKEY に対応する値を指定する。 {@KEY} に置き換わる。
	private String[]  footerKeys	;				// 繰り返し部の終了後に表示する key 部分を指定する。カンマで複数指定できる。
	private String[]  footerVals	;				// 繰り返し部の終了後に表示する key に対する値を指定する。
	private boolean   pageEndCut	= true;			// ボディー部(繰り返し部)がなくなったときに、それ以降のページを出力するか指定する。
	private String    fileURL		= BASE_URL;		// 雛型のHTMLファイルの保存してある ディレクトリを指定します。
	private String    outFileURL	= BASE_URL;		// 出力HTMLファイルの保存してある ディレクトリを指定します。
	private String    outFilename	;				// 出力HTMLファイル名を指定します。 ディレクトリ名を含んでも構いません。
	private String    reportClass	= "HTML";

	private transient DBTableModel table ;
	private String  tableId			= HybsSystem.TBL_MDL_KEY ;
	// 3.8.0.4 (2005/08/08) 帳票出力に準拠した方式に変更
	private String  listId			;				// 雛型のHTMLファイル名を指定します。
	private String  programFile		;				// HTMLファイルのEXCEL化を行うバッチファイルを指定します。

	/**
	 * Taglibの終了タグが見つかったときに処理する doEndTag() を オーバーライドします。
	 *
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。release2() を doEndTag()で呼ぶ。
	 * @og.rev 3.8.0.4 (2005/08/08) 帳票出力に準拠した方式に変更
	 * @og.rev 4.0.0.0 (2007/10/18) メッセージリソース統合( getResource().getMessage ⇒ getResource().getLabel )
	 * @og.rev 4.0.0.0 (2007/11/28) メソッドの戻り値をチェックします。
	 *
	 * @return	後続処理の指示
	 */
	@Override
	public int doEndTag() {
		debugPrint();

		final int rtnCode;

		table = (DBTableModel)getObject( tableId );
		if( table == null || table.getRowCount() == 0 ) {
			rtnCode = SKIP_PAGE ;		// ページの残りの処理を行わない。
		}
		else {
			synchronized( ReportTableTag.class ) {
				final String reportDir  = HybsSystem.url2dir( REPORT_URL ) + SYSTEM_ID + FS + listId ;
		//		String ykno       = HybsSystem.getDate( "yyyyMMddHHmmss" );
				final String ykno       = String.valueOf( Math.round( Math.random() * 1000000 ) ) ;

				create( reportDir,ykno ) ;

				// 処理クラス(reportClass)が HTML の場合は、Shell を起動して、VBS で EXCEL化が必要。
				if( programFile != null && "HTML".equalsIgnoreCase( reportClass ) ) {
					final String htmlFile = reportDir + FS + ykno ;
					final String cmd = makeShellCommand( htmlFile,listId );
					programRun( cmd );
				}

				if( outFilename != null ) {
					final File xlsFile = new File( reportDir,ykno + ".xls" );
					final File outDir  = new File( HybsSystem.url2dir( outFileURL ) );
					if( !outDir.exists() && !outDir.mkdirs() ) {
						final String errMsg = "所定のフォルダを作成できませんでした。[" + outDir + "]" ;
						throw new RuntimeException( errMsg );
					}
					final File outFile = new File( outDir,outFilename );
					FileUtil.copy(  xlsFile , outFile );

					final String msg = getResource().getLabel( "MSG0003" ) 		// MSG0003=ファイルの登録が完了しました。
								+ BR
								+ getResource().getLabel( "MSG0022" ) 	// MSG0022=ファイル名
								+ ":" + outFile.getAbsolutePath() ;
					jspPrint( msg );
				}
			}
			rtnCode = EVAL_PAGE ;
		}

		return rtnCode ;
	}

	/**
	 * タグリブオブジェクトをリリースします。
	 * キャッシュされて再利用されるので、フィールドの初期設定を行います。
	 *
	 * @og.rev 2.0.0.4 (2002/09/27) カスタムタグの release() メソッドを、追加
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。release2() を doEndTag()で呼ぶ。
	 * @og.rev 3.1.3.0 (2003/04/10) REPORT_ENCODE(帳票エンコーディング)を利用するように修正。
	 * @og.rev 3.8.0.4 (2005/08/08) filename 削除、listId ,programFile 追加 ,
	 *
	 */
	@Override
	protected void release2() {
		super.release2();
		headerKeys		= null;
		headerVals		= null;
		footerKeys		= null;
		footerVals		= null;
		pageEndCut		= true;
		fileURL			= BASE_URL;
		outFileURL		= BASE_URL;
		outFilename		= null;			// 出力ファイル名
		reportClass		= "HTML";
		table			= null;
		tableId			= HybsSystem.TBL_MDL_KEY ;
		listId			= null ;		// 3.8.0.4 (2005/08/08)
		programFile		= null ;		// 3.8.0.4 (2005/08/08)
	}

	/**
	 * TableWriter の実オブジェクトを生成して，PrintWriter に書き込みます。
	 *
	 * @og.rev 3.1.3.0 (2003/04/10) REPORT_ENCODE(帳票エンコーディング)を利用するように修正。
	 * @og.rev 3.5.4.3 (2004/01/05) HTMLDBTableReport のクラス名変更。
	 * @og.rev 3.6.0.0 (2004/09/17) メソッド名の変更。setInputFile ⇒ setTemplateFile
	 * @og.rev 3.8.0.0 (2005/06/07) setTemplateFile メソッドの引数を String  ⇒ File に変更
	 * @og.rev 3.8.0.4 (2005/08/08) 帳票出力に準拠した方式に変更
	 * @og.rev 4.0.0.0 (2005/01/31) lang ⇒ ResourceManager へ変更
	 *
	 * @param	reportDir	出力ディレクトリ名
	 * @param	ykno		要求番号
	 */
	private void create( final String reportDir,final String ykno )  {

		final String MODELDIR = HybsSystem.url2dir( fileURL ) ;

		File templateFile      = null;
		File firstTemplateFile = null;

		// 本来は、各クラス中で処理すべき。そのためのオブジェクト指向なのだから。
		if( "HTML".equalsIgnoreCase( reportClass ) ) {
			templateFile      = FileUtil.checkFile( MODELDIR, listId + ".html" , 1 );
			firstTemplateFile = FileUtil.checkFile( MODELDIR, listId + "_FIRST.html" ,1 );
		}
		else if( "Excel".equalsIgnoreCase( reportClass ) ) {
			templateFile      = FileUtil.checkFile( MODELDIR, listId + ".xls" , 1 );
		}
		else {
			final String errMsg = "リポートクラスがサポート外です。[" + reportClass + "]"
						+ "クラスは、HTML、Excel のみサポートされています。" ;
			throw new RuntimeException( errMsg );
		}

		FileUtil.copy( templateFile,new File( reportDir ) );

		final String className = "org.opengion.hayabusa.report.DBTableReport_" + reportClass  ;
		final DBTableReport report = (DBTableReport)HybsSystem.newInstance( className );	// 3.5.5.3 (2004/04/09)

		report.setDBTableModel( table );
		report.setTemplateFile( templateFile );	// 3.6.0.0 (2004/09/17)
		report.setFirstTemplateFile( firstTemplateFile );	// 3.6.0.0 (2004/09/17)
		report.setOutputDir( reportDir );
		report.setOutputFileKey( ykno );
		report.setHeaderKeys( headerKeys );
		report.setHeaderVals( headerVals );
		report.setFooterKeys( footerKeys );
		report.setFooterVals( footerVals );
		report.setPageEndCut( pageEndCut );
		report.setResourceManager( getResource() );	// 4.0.0 (2005/01/31)
		report.setListId( listId );		// 3.6.1.0 (2005/01/05)
		report.writeReport();
	}

	/**
	 * 【TAG】(通常は使いません)結果のDBTableModelを、sessionに登録するときのキーを指定します
	 *		(初期値:HybsSystem#TBL_MDL_KEY[={@og.value org.opengion.hayabusa.common.HybsSystem#TBL_MDL_KEY}])。
	 *
	 * @og.tag
	 * 検索結果より、DBTableModelオブジェクトを作成します。これを、下流のviewタグ等に
	 * 渡す場合に、通常は、session を利用します。その場合の登録キーです。
	 * query タグを同時に実行して、結果を求める場合、同一メモリに配置される為、
	 * この tableId 属性を利用して、メモリ空間を分けます。
	 *		(初期値:HybsSystem#TBL_MDL_KEY[={@og.value org.opengion.hayabusa.common.HybsSystem#TBL_MDL_KEY}])。
	 *
	 * @param	id テーブルID (sessionに登録する時のID)
	 */
	public void setTableId( final String id ) {
		tableId = nval( getRequestParameter( id ), tableId );
	}

	/**
	 * 【TAG】帳票IDを指定します。
	 *
	 * @og.tag
	 * 雛形ファイルは、帳票ID.html となります。また、ファーストページ対応の場合は、
	 * 帳票ID_FIRST.html になります。
	 * なお、filename 属性が指定された場合は、そちらが優先されます。
	 *
	 * @og.rev 3.8.0.4 (2005/08/08) 新規追加
	 *
	 * @param	listId	帳票ID
	 */
	public void setListId( final String listId ) {
		this.listId = nval( getRequestParameter( listId ), this.listId );
	}

	/**
	 * 【TAG】固定部の{&#064;KEY} の KEY 部分をCSV形式で複数指定します。
	 *
	 * @og.tag
	 * カンマで複数指定できます。
	 * 分解方法は、CSV変数を先に分解してから、getRequestParameter で値を取得します。
	 * こうしないとデータ自身にカンマを持っている場合に分解をミスる為です。
	 *
	 * @og.rev 3.5.6.2 (2004/07/05) 先に配列に分解してからリクエスト変数の値を取得
	 *
	 * @param   hKeys 固定部のkey
	 */
	public void setHeaderKeys( final String hKeys ) {
		headerKeys = getCSVParameter( hKeys );
	}

	/**
	 * 【TAG】固定部のKEY に対応する値をCSV形式で複数指定します。
	 *
	 * @og.tag
	 * カンマで複数指定で、リクエスト情報でも設定できます。
	 * 分解方法は、CSV変数を先に分解してから、getRequestParameter で値を取得します。
	 * こうしないとデータ自身にカンマを持っている場合に分解をミスる為です。
	 *
	 * @og.rev 3.5.6.2 (2004/07/05) 先に配列に分解してからリクエスト変数の値を取得
	 *
	 * @param   hVals 固定部の値
	 */
	public void setHeaderVals( final String hVals ) {
		headerVals = getCSVParameter( hVals );
	}

	/**
	 * 【TAG】繰り返し部の終了後に表示する key 部分をCSV形式で複数指定します。
	 *
	 * @og.tag
	 * カンマで複数指定できます。
	 * 分解方法は、CSV変数を先に分解してから、getRequestParameter で値を取得します。
	 * こうしないとデータ自身にカンマを持っている場合に分解をミスる為です。
	 *
	 * @og.rev 3.5.6.2 (2004/07/05) 先に配列に分解してからリクエスト変数の値を取得
	 *
	 * @param   ftKeys 繰り返し部の終了後に表示するkey
	 */
	public void setFooterKeys( final String ftKeys ) {
		footerKeys = getCSVParameter( ftKeys );
	}

	/**
	 * 【TAG】固定部のKEY に対応する値をCSV形式で複数指定します。
	 *
	 * @og.tag
	 * カンマで複数指定で、リクエスト情報でも設定できます。
	 * 分解方法は、CSV変数を先に分解してから、getRequestParameter で値を取得します。
	 * こうしないとデータ自身にカンマを持っている場合に分解をミスる為です。
	 *
	 * @og.rev 3.5.6.2 (2004/07/05) 先に配列に分解してからリクエスト変数の値を取得
	 * @og.rev 3.8.1.2 (2005/12/19) footer 関連の値とpageEndCut の関係を解除します。
	 *
	 * @param   ftVals 繰り返し部の終了後に表示する値
	 */
	public void setFooterVals( final String ftVals ) {
		footerVals = getCSVParameter( ftVals );
	}

	/**
	 * 【TAG】ページエンドカットするかどうか[true:する/false:しない]を指定します(初期値:true:カットする)。
	 *
	 * @og.tag
	 * ページエンドカットとは、ボディー部(繰り返し部)がなくなったときに、それ以降の表示を打ち切る（カットする）
	 * 機能で、(true:カットする) を指定すると、それ以降を出力しません。
	 * 初期値は "true" (なくなった時点で、出力しない)です。
	 *
	 * @og.rev 3.8.1.2 (2005/12/19) footer 関連の値とpageEndCut の関係を解除します。
	 *
	 * @param   peCut ページ終了カット [true:する/false:しない]
	 */
	public void setPageEndCut( final String peCut ) {
		pageEndCut = nval( getRequestParameter( peCut ),pageEndCut );
	}

	/**
	 * 【TAG】雛型のHTMLファイルの保存してある ディレクトリを指定します
	 *		(初期値:FILE_URL[={@og.value org.opengion.hayabusa.common.SystemData#FILE_URL}])。
	 *
	 * @og.tag
	 * この属性で指定されるディレクトリのファイルを読み取ります。
	 * 指定方法は、通常の fileURL 属性と同様に、先頭が、'/' (UNIX) または、２文字目が、
	 * ":" (Windows)の場合は、指定のURLそのままのディレクトリに、そうでない場合は、
	 * システムパラメータ の FILE_URL 属性で指定のフォルダの下に、作成されます。
	 * fileURL = "{&#064;USER.ID}" と指定すると、FILE_URL 属性で指定のフォルダの下に、
	 * さらに、各個人ID別のフォルダを作成して、そこを操作します。
	 * (初期値:FILE_URL[={@og.value org.opengion.hayabusa.common.SystemData#FILE_URL}])。
	 *
	 * @og.rev 4.0.0.0 (2005/01/31) StringUtil.urlAppend メソッドの利用
	 * @og.rev 4.0.0.0 (2007/11/20) 指定されたディレクトリ名の最後が"\"or"/"で終わっていない場合に、"/"を付加する。
	 *
	 * @param	url 雛型のHTMLファイルのディレクトリ
	 */
	public void setFileURL( final String url ) {
		String furl = nval( getRequestParameter( url ),null );
		if( furl != null ) {
			final char ch = furl.charAt( furl.length()-1 );
			if( ch != '/' && ch != '\\' ) { furl = furl + "/"; }
			fileURL = StringUtil.urlAppend( fileURL,furl );
		}
	}

	/**
	 * 【TAG】HTMLファイルをEXCEL変換する場合に使用するBATファイルを指定します。
	 *
	 * @og.tag
	 * ファイルは、フルパスで指定してください。
	 * 指定がない場合は、変換処理は行いません。
	 * 通常は、dbdef2/def/Script/runExcelPrint.bat を呼び出してください。
	 * 初期値は、null(変換処理しない)です。
	 *
	 * @og.rev 3.8.0.4 (2005/08/08) 新規追加
	 *
	 * @param   programFile プログラムファイル名
	 */
	public void setProgramFile( final String programFile ) {
		this.programFile = nval( getRequestParameter( programFile ),this.programFile );
	}

	/**
	 * 【TAG】出力HTMLファイルの保存してあるディレクトリを指定します
	 *		(初期値:FILE_URL[={@og.value org.opengion.hayabusa.common.SystemData#FILE_URL}])。
	 *
	 * @og.tag
	 * この属性で指定されるディレクトリにファイルを出力します。
	 * 指定方法は、通常の fileURL 属性と同様に、先頭が、'/' (UNIX) または、２文字目が、
	 * ":" (Windows)の場合は、指定のURLそのままのディレクトリに、そうでない場合は、
	 * システムパラメータ の FILE_URL 属性で指定のフォルダの下に、作成されます。
	 * fileURL = "{&#064;USER.ID}" と指定すると、FILE_URL 属性で指定のフォルダの下に、
	 * さらに、各個人ID別のフォルダを作成して、そこに出力します。
	 * (初期値:FILE_URL[={@og.value org.opengion.hayabusa.common.SystemData#FILE_URL}])。
	 *
	 * @og.rev 4.0.0.0 (2007/11/20) 指定されたディレクトリ名の最後が"\"or"/"で終わっていない場合に、"/"を付加する。
	 *
	 * @param	url 出力HTMLファイルのディレクトリ
	 */
	public void setOutFileURL( final String url ) {
		String furl = nval( getRequestParameter( url ),null );
		if( furl != null ) {
			final char ch = furl.charAt( furl.length()-1 );
			if( ch != '/' && ch != '\\' ) { furl = furl + "/"; }
			outFileURL = StringUtil.urlAppend( outFileURL,furl );
		}
	}

	/**
	 * 【TAG】ファイルを作成するときの出力ファイル名をセットします。
	 *
	 * @og.tag ファイルを作成するときの出力ファイル名をセットします。
	 *
	 * @param   filename 出力ファイル名
	 */
	public void setOutFilename( final String filename ) {
		this.outFilename = nval( getRequestParameter( filename ),this.outFilename );
	}

	/**
	 * 【TAG】実際に書き出すクラス名の略称(DBTableReport_**** の ****)をセットします(初期値:HTML)。
	 *
	 * @og.tag
	 * これは、org.opengion.hayabusa.report 以下の DBTableReport_**** クラスの **** を
	 * 与えます。これらは、DBTableReport インターフェースを継承したサブクラスです。
	 * 初期値は、"HTML" です。
	 * 属性クラス定義の {@link org.opengion.hayabusa.report.DBTableReport DBTableReport} を参照願います。
	 *
	 * @param   reportClass クラス名(の略称)
	 * @see		org.opengion.hayabusa.report.DBTableReport  DBTableReportのサブクラス
	 */
	public void setReportClass( final String reportClass ) {
		this.reportClass = nval( getRequestParameter( reportClass ),this.reportClass );
	}

	/**
	 * シェルコマンドの文字列を作成します。
	 *
	 * 処理クラス(reportClass)が HTML の場合は、Shell を起動して、VBS で EXCEL化が必要。
	 *
	 * @og.rev 3.8.0.4 (2005/08/08) 新規追加
	 * @og.rev 3.8.0.8 (2005/10/03) デーモングループのデフォルト値設定
	 *
	 * @param	htmlFile	パーサ済みのHTMLファイル(拡張子なし)
	 * @param	listId	雛形ファイル(帳票ID)
	 *
	 * @return シェルコマンドの文字列
	 * @og.rtnNotNull
	 */
	private String makeShellCommand( final String htmlFile,final String listId ) {
		final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE )
			.append( programFile ).append( ' ' )		// 実行するコマンド
			.append( '"' )								// 6.0.2.5 (2014/10/31) char を append する。
			.append( htmlFile ).append( "_*.html\" " )	// 入力HTMLファイル
			.append( "\"xls\" " )						// プリンタ名
			// ダミーファイルを指定する必要がある。
			.append( '"' )								// 6.0.2.5 (2014/10/31) char を append する。
			.append( htmlFile ).append( ".xls\" " )		// ダミー出力ファイル
			.append( listId )							// モデルファイル名
			.append( " NULL" );							// ダミーデーモングループ

		return buf.toString();
	}

	/**
	 * 実際のレポート出力処理を行います。
	 *
	 * 処理クラス(reportClass)が HTML の場合は、Shell を起動して、VBS で EXCEL化が必要。
	 *
	 * @og.rev 3.8.0.4 (2005/08/08) 新規追加
	 *
	 * @param	shellCmd	シェルを実行するコマンド文字列
	 */
	private void programRun( final String shellCmd ) {
		final Shell shell = new Shell();
		shell.setCommand( shellCmd,true );		// BATCHプロセスで実行する
		shell.setWait( true );					// プロセスの終了を待つ
		shell.setTimeout( TIMEOUT );			// 3.6.1.0 (2005/01/05) Shell の タイムアウトを設定

		final int rtnCode = shell.exec();				// 0 は正常終了を示す

		if( rtnCode != 0 ) {
			final String errMsg = "Shell Command exequte Error." + CR
						+ "=============================="
						+ shellCmd + CR
						+ shell.getStdoutData() + CR
						+ shell.getStderrData() + CR ;
			throw new HybsSystemException( errMsg );		// 3.5.5.4 (2004/04/15) 引数の並び順変更
		}
	}

	/**
	 * タグの名称を、返します。
	 * 自分自身のクラス名より、自動的に取り出せないため、このメソッドをオーバーライドします。
	 *
	 * @og.rev 4.0.0.0 (2005/01/31) 新規追加
	 *
	 * @return  タグの名称
	 * @og.rtnNotNull
	 */
	@Override
	protected String getTagName() {
		return "report" ;
	}

	/**
	 * シリアライズ用のカスタムシリアライズ書き込みメソッド。
	 *
	 * @og.rev 4.0.0.0 (2006/09/31) 新規追加
	 * @serialData 一部のオブジェクトは、シリアライズされません。
	 *
	 * @param	strm	ObjectOutputStreamオブジェクト
	 * @throws IOException	入出力エラーが発生した場合
	 */
	private void writeObject( final ObjectOutputStream strm ) throws IOException {
		strm.defaultWriteObject();
	}

	/**
	 * シリアライズ用のカスタムシリアライズ読み込みメソッド
	 *
	 * ここでは、transient 宣言された内部変数の内、初期化が必要なフィールドのみ設定します。
	 *
	 * @og.rev 4.0.0.0 (2006/09/31) 新規追加
	 * @serialData 一部のオブジェクトは、シリアライズされません。
	 *
	 * @param	strm	ObjectInputStreamオブジェクト
	 * @see #release2()
	 * @throws IOException	シリアライズに関する入出力エラーが発生した場合
	 * @throws ClassNotFoundException	クラスを見つけることができなかった場合
	 */
	private void readObject( final ObjectInputStream strm ) throws IOException , ClassNotFoundException {
		strm.defaultReadObject();
	}

	/**
	 * このオブジェクトの文字列表現を返します。
	 * 基本的にデバッグ目的に使用します。
	 *
	 * @return このクラスの文字列表現
	 * @og.rtnNotNull
	 */
	@Override
	public String toString() {
		return ToString.title( this.getClass().getName() )
				.println( "VERSION"		,VERSION	)
				.println( "headerKeys"		,headerKeys		)
				.println( "headerVals"		,headerVals		)
				.println( "footerKeys"		,footerKeys		)
				.println( "footerVals"		,footerVals		)
				.println( "pageEndCut"		,pageEndCut		)
				.println( "fileURL"			,fileURL		)
				.println( "outFileURL"		,outFileURL		)
				.println( "outFilename"		,outFilename	)
				.println( "reportClass"		,reportClass	)
				.println( "tableId"			,tableId		)
				.println( "listId"			,listId			)
				.println( "programFile"		,programFile	)
				.println( "REPORT_FILE_URL"	,REPORT_URL		)
				.println( "SYSTEM_ID"		,SYSTEM_ID		)
				.println( "TIMEOUT"			,TIMEOUT		)
				.println( "BASE_URL"		,BASE_URL		)
				.println( "Other..."	,getAttributes().getAttribute() )
				.fixForm().toString() ;
	}
}
